// an implementation of a general graph with nodes and directed edges

import java.util.*;

public class Graph {
    //--------------------------------------------------------------------
    // new method
    public void addVertex(String label, double x, double y) {
        // make sure label is unique
        if (this.labelMap.containsKey(label)) {
            System.out.printf("ERROR: Vertex %s already exists\n", label);
        } else {
            Vertex v = new Vertex(label, x, y);
            this.allVertices.add(v);
            this.labelMap.put(label, v);
        }
    }
    //--------------------------------------------------------------------

    // instance variables
    private List<Vertex> allVertices;     // all vertices in the graph
    private Map<String,Vertex> labelMap;  // maps vertex labels to vertices

    // constructor
    public Graph() {
        this.allVertices = new ArrayList<Vertex>();
        this.labelMap = new HashMap<String,Vertex>();
    }

    // returns a list of all vertices in the graph
    public List<Vertex> getVertices() {
        return this.allVertices;
    }

    // returns the vertex with the given label
    public Vertex getVertex(String label) {
        return this.labelMap.get(label);
    }

    // adds a vertex with the given label to the graph
    public void addVertex(String label) {
        // make sure label is unique
        if (this.labelMap.containsKey(label)) {
            System.out.printf("ERROR: Vertex %s already exists\n", label);
        } else {
            Vertex v = new Vertex(label);
            this.allVertices.add(v);
            this.labelMap.put(label, v);
        }
    }

    // returns true if a vertex with the specified label exists
    public boolean containsVertex(String label) {
        return this.labelMap.containsKey(label);
    }

    // resets all vertices to the unvisited state
    public void clearVisited() {
        for (Vertex v : this.allVertices) {
            v.setVisited(false);
        }
    }

    // unlabled directed edge
    public void addDirectedEdge(String label1, String label2) {
        Vertex v1 = this.getVertex(label1);
        Vertex v2 = this.getVertex(label2);
        Edge e = new Edge(v1, v2);
        v1.addOutgoingEdge(e);
    }

    // directed edge labeled with a cost value
    public void addDirectedEdge(String label1, String label2, double cost) {
        Vertex v1 = this.getVertex(label1);
        Vertex v2 = this.getVertex(label2);
        Edge e = new Edge(v1, v2, cost);
        v1.addOutgoingEdge(e);
    }

    // unlabeled undirected edge
    public void addUndirectedEdge(String label1, String label2) {
        this.addDirectedEdge(label1, label2);
        this.addDirectedEdge(label2, label1);
    }

    // undirected edge labeled with a cost value
    public void addUndirectedEdge(String label1, String label2, double cost) {
        this.addDirectedEdge(label1, label2, cost);
        this.addDirectedEdge(label2, label1, cost);
    }

    // returns a list of all edges in the graph
    public List<Edge> getEdges() {
        List<Edge> edges = new ArrayList<Edge>();
        for (Vertex v : this.allVertices) {
            edges.addAll(v.outgoingEdges());
        }
        return edges;
    }

    // returns the sum of the costs of all edges
    public double totalEdgeCost() {
        double total = 0;
        for (Edge e : this.getEdges()) {
            total = total + e.getCost();
        }
        return total;
    }

    // returns a string representation of the graph
    public String toString() {
        String s = "";
        for (Vertex v : this.allVertices) {
            s += "Vertex: " + v + "\n";
            for (Edge e : v.outgoingEdges()) {
                s += "   " + e + "\n";
            }
        }
        return s;
    }
}
